/*
SUPPORT: sinass Sharma, Ashish
*/
/****************************************************************************** 
 * Job:             Load_STG2DM_Geo_prod_price_fact                           * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Tables:   PRICE_FACT_HISTORY -                                      * 
 *                   DI_DM.PRICE_FACT_HISTORY                                 * 
 *                  PRICE_FACT_CURRENT -                                      * 
 *                   DI_DM.PRICE_FACT_CURRENT                                 * 
 *                  PRICE_FACT_FUTURE  -                                      * 
 *                   DI_DM.PRICE_FACT_FUTURE                                  * 
 *                  PROD_HIER_SK - DI_DM.PRODUCT_DM                           * 
 *                  GEOGRAPHY_DM - DI_DM.GEOGRAPHY_DM                         * 
 *                  STG_GEO_PROD_PRICE -                                      * 
 *                   DIDM_STG.STG_GEO_PROD_PRICE                              * 
 * Target Tables:   PRICE_FACT_HISTORY -                                      * 
 *                   DI_DM.PRICE_FACT_HISTORY                                 * 
 *                  PRICE_FACT_CURRENT -                                      * 
 *                   DI_DM.PRICE_FACT_CURRENT                                 *
 *                  PRICE_FACT_FUTURE  -                                      * 
 *                   DI_DM.PRICE_FACT_FUTURE                                  * 
 *                                                                            * 
 * Generated on:    March 6, 2013                                             * 
 ******************************************************************************/ 

/* General macro variables  */ 
%let etls_jobName = %nrquote(Load_STG2DM_Geo_prod_price_fact);
%let etls_userID = %nrquote(sinass@row);

/* Setup to capture return codes  */ 
%global job_rc trans_rc sqlrc; 
%let sysrc = 0;
%let job_rc = 0;
%let trans_rc = 0;
%let sqlrc = 0;
%global syserr; 

%macro rcSet(error); 
   %if (&error gt &trans_rc) %then 
      %let trans_rc = &error;
   %if (&error gt &job_rc) %then 
      %let job_rc = &error;
%mend rcSet; 

%macro rcSetDS(error); 
   if &error gt input(symget('trans_rc'),12.) then 
      call symput('trans_rc',trim(left(put(&error,12.))));
   if &error gt input(symget('job_rc'),12.) then 
      call symput('job_rc',trim(left(put(&error,12.))));
%mend rcSetDS; 

/* Create metadata macro variables */
%let IOMServer      = %nrquote(MIMain);

/* Setup for capturing job status  */ 
%let etls_startTime = %sysfunc(datetime(),datetime.);
%let etls_recordsBefore = 0;
%let etls_recordsAfter = 0;
%let etls_lib = 0;
%let etls_table = 0;

%global etls_debug; 
%macro etls_setDebug; 
   %if %str(&etls_debug) ne 0 %then 
      OPTIONS MPRINT%str(;); 
%mend; 
%etls_setDebug; 

/*---- Start of Pre-Process Code  ----*/ 
/*Fetch the value of loadview pararmeter for the job LOAD_SRC2STG_GEO_PROD_PRICE*/
%etl_initjobparam (monlib=di_mon, jobname=%nrquote(LOAD_SRC2STG_GEO_PROD_PRICE));

%etl_initjobparam (monlib=di_mon, jobname=%nrquote(&etls_jobName));

/*---- End of Pre-Process Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW-SetVariables                                           * 
 * Transform:       User Written                                              * 
 * User Written:    SetVar Setting required variables                         * 
 *============================================================================*/ 

%macro etl_check_exist;
   %local l_current_date_sk etl_msg abort date_dm_obs l_loadview;
   %let abort = 0 ;

   /* Abort if current_date_sk is null */
   proc sql noprint;
      select spec_value
      into :l_current_date_sk
      from di_dm.global_setting
      where spec_nm="CURRENT_DATE_SK";
   quit;

   %let l_current_date_sk = &l_current_date_sk;
   %if (&l_current_date_sk eq %str(0) or &l_current_date_sk eq %str() or &l_current_date_sk eq %str(.)) %then %do;  
      %let etl_msg = %etl_msg(mid=5012,args=CURRENT_DATE_SK GLOBAL_SETTING);
      %put &etl_msg;
      %let abort = 1 ;
   %end;

   /* Check for existence of DI_DM.DATE_DM dataset */
   %if NOT %sysfunc(exist(DI_DM.DATE_DM)) %then %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM.DATE_DM);
      %put &etl_msg;
      %let abort = 2 ;
   %end;
   %else %do;
      %let date_dm_obs = %di_util_nobs(DI_DM.DATE_DM);
      %if &date_dm_obs = 0 %then %do;
         %let etl_msg = %etl_msg(mid=5008,args=DI_DM.DATE_DM);
         %put &etl_msg;
      %end;
   %end;

   /* Check for existence of DI_DM.GEOGRAPHY_DM dataset */
   %if NOT %sysfunc(exist(DI_DM.GEOGRAPHY_DM)) %then %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM.GEOGRAPHY_DM);
      %put &etl_msg;
      %let abort = 3 ;
   %end;


   /* Check for existence of DI_DM.PRODUCT_DM dataset */
   %if NOT %sysfunc(exist(DI_DM.PRODUCT_DM)) %then %do;
      %let etl_msg = %etl_msg(mid=5005,args=DI_DM.PRODUCT_DM);
      %put &etl_msg;
      %let abort = 4 ;
   %end;

   
   /*--select data_lvl_id from global_setting table--*/
   proc sql noprint;
      select spec_value into :l_gp_low_data_lvl 
      from di_dm.global_setting
      where upcase(spec_nm)='GEO_PROD_LOW_DATA_LVL';
   quit;

   %if &l_gp_low_data_lvl eq %str() %then %do; 
      %let etl_msg = %etl_msg(mid=5012,args=GEO_PROD_LOW_DATA_LVL GLOBAL_SETTING);
      %put &etl_msg;
      %let abort = 5 ;
   %end;
   %else %do;/* check for validity of global_setting */
     %global l_prod_lvl l_geo_lvl;
         proc sql noprint;
         select Prod_lvl , geo_lvl into :l_prod_lvl , :l_geo_lvl 
           from di_dm.data_lvl
          where data_lvl_id = &l_gp_low_data_lvl;
      quit;
      %if &l_prod_lvl eq %str() %then %do;
          %let etl_msg = %etl_msg(mid=5012,args=GLOBAL_SETTING.GEO_PROD_LOW_DATA_LVL DI_DM.DATA_LVL);
          %put &etl_msg;
          %let abort = 6 ;
      %end;    
   %end;
   %put ______: loadview=&loadview;
 
   %if &loadview eq 1 %then %do;
      %if NOT %sysfunc(exist(didm_stg.V_stg_geo_prod_price, view)) %then %do;
         %let etl_msg = %etl_msg(mid=5005,args=didm_stg.V_stg_geo_prod_price);
         %put &etl_msg;
         %let abort = 7 ;
      %end;
   %end;
   %else %if &loadview ne 1 and &loadview ne 0 %then %do;
      %let etl_msg = %etl_msg(mid=5012,args=LOADVIEW JOB_PARAM);
      %put &etl_msg;
      %let abort = 8;
   %end;
 

   /*---- Read current_date_sk ---------*/
   %global wk_end_dt wk_start_dt;

   %let wk_start_dt=0;
   %let wk_end_dt=0;

   proc sql noprint;
      select compress(put(fiscal_start_dt,best.)),
             compress(put(fiscal_end_dt,best.))
        into :wk_start_dt,
             :wk_end_dt
        from di_dm.date_dm
       where date_sk = %eval(&l_current_date_sk - 1);
   quit;

   %let wk_start_dt=&wk_start_dt;
   %let wk_end_dt=&wk_end_dt;

   %put ______: wk_start_dt=&wk_start_dt;
   %put ______: wk_end_dt=&wk_end_dt;

   %if %eval(&wk_start_dt le 0) %then %do;
       %let etl_msg = %etl_msg(mid=5012,args=CURRENT_DATE_SK-1 DATE_DM);
       %put &etl_msg;
       %let abort = 9 ;
   %end;
   %put ______: etlp_reload_history_data = &etlp_reload_history_data;

   %if not %symexist(etlp_move_futr_to_curr) %then %do;
      %let etl_msg = %etl_msg(mid=5012,args=etlp_move_futr_to_curr JOB_PARAM);
      %put &etl_msg;
      %let abort = 10 ;   
   %end;
   
   %if not %symexist(stg_data_override) %then %do;
      %let etl_msg = %etl_msg(mid=5012,args=stg_data_override JOB_PARAM);
      %put &etl_msg;
      %let abort = 11 ;   
   %end;   
   
   %if &abort gt 0 %then %do;
      %etl_abort;
   %end;

   %put ______: etlp_move_futr_to_curr=&etlp_move_futr_to_curr;
   %put ______: stg_data_override=&stg_data_override;
   
   %rcSet(&syserr); 
   %rcSet(&sqlrc); 

   %global err_table src_cnt;

   /* Check if library trg_lib exists. If not, then assign di_dm to trg_lib */
   %if %sysfunc(libref(trg_lib)) ne 0 %then %do;
      libname trg_lib (di_dm);
   %end;
   /* Check if library stg_lib exists. If not, then assign didm_stg to stg_lib */
   %if %sysfunc(libref(stg_lib)) ne 0 %then %do;
      libname stg_lib (didm_stg);
         %if NOT %symexist(src_table) %then
            %global src_table;
         %if "&src_table" eq "" %then %do;
            %if &loadview eq 0 %then  
               %let src_table=stg_geo_prod_price;
            %else %if &loadview eq 1 %then  
               %let src_table=V_stg_geo_prod_price;
         %end;
   %end;

   %put ______: src_table=&src_table;

   /* Check if library err_lib exists. If not, then assign di_mon to err_lib */

   %if (%sysfunc(libref(err_lib)) eq 0) AND (%symexist(part_sk)) %then %do;
      %let err_table=STG_GEO_PROD_PRICE_X_&part_sk;
   %end;
   %else %do;
      libname err_lib (di_mon);
      %let err_table=STG_GEO_PROD_PRICE_X;
   %end;

   %put ______: err_table=&err_table;
   /* Check existense of target table */
   %if not %sysfunc( exist(trg_lib.PRICE_FACT_CURRENT)) %then %do;
   data trg_lib.PRICE_FACT_CURRENT;
      set di_dm.PRICE_FACT_CURRENT;
      stop;
   run;
   %end;

   %if not %sysfunc( exist(trg_lib.PRICE_FACT_HISTORY)) %then %do;
      data trg_lib.PRICE_FACT_HISTORY;
         set di_dm.PRICE_FACT_HISTORY;
         stop;
      run;
   %end;

   %if not %sysfunc( exist(trg_lib.PRICE_FACT_FUTURE)) %then %do;
      data trg_lib.PRICE_FACT_FUTURE;
         set di_dm.PRICE_FACT_FUTURE;
         stop;
      run;
   %end;

   %global etls_recordsBefore_pfc etls_recordsAfter_pfc etls_recordsBefore_pfh etls_recordsAfter_pfh etls_recordsBefore_pff etls_recordsAfter_pff;
   %let etls_recordsBefore_pfc=0;    
   %let etls_recordsAfter_pfc=0; 
   %let etls_recordsBefore_pfh=0;    
   %let etls_recordsAfter_pfh=0; 
   %let etls_recordsBefore_pff=0;    
   %let etls_recordsAfter_pff=0; 
 

   /* Get record count from PRICE_FACT_HISTORY */
   %let etls_recordsBefore_pfh=%di_util_nobs(trg_lib.PRICE_FACT_HISTORY);
   /* Get record count from PRICE_FACT_CURRENT */
   %let etls_recordsBefore_pfc=%di_util_nobs(trg_lib.PRICE_FACT_CURRENT);
   /* Get record count from PRICE_FACT_FUTURE */
   %let etls_recordsBefore_pff=%di_util_nobs(trg_lib.PRICE_FACT_FUTURE);


   /* Create Job_status in err_lib if not exist */
   %if not %sysfunc( exist(err_lib.job_status)) %then %do;
      %di_util_lock(di_mon.job_status);
      data err_lib.job_status;
         set di_mon.job_status;
         stop;
      run;
      %di_util_unlock(di_mon.job_status);
   %end;

   /* Create error table in err_lib if not exist */
   %if not %sysfunc( exist(err_lib.&err_table)) %then %do;
      %di_util_lock(di_mon.STG_GEO_PROD_PRICE_X);
      data err_lib.&err_table;
         set di_mon.STG_GEO_PROD_PRICE_X;
         stop;
      run;
      %di_util_unlock(di_mon.STG_GEO_PROD_PRICE_X);
   %end;

   /* Set exception table and exception records for job_status macro */
   %let etlp_exception_table=ERR_LIB.&err_table; 
   %put ______: etlp_exception_table=&etlp_exception_table;

   %di_util_lock(&etlp_exception_table);
   %global len_excp_msg1;
   data _null_;
      dsid=open("&etlp_exception_table","I", ,'D');
      num=attrn(dsid,"nvars");
      do i=1 to num;
         name=varname(dsid,i);
         if upcase(name) = 'ETLS_EXCEPTION_COND' then do;
            len_excp_msg1 = varlen(dsid,i);
            call symput('len_excp_msg1',compress(len_excp_msg1));
         end;
      end;
      rc=close(dsid);
   run;
   %di_util_unlock(&etlp_exception_table);
   %let len_excp_msg1=&len_excp_msg1;
   %put ______: len_excp_msg1=&len_excp_msg1;

   %if %eval(&len_excp_msg1 le 0) %then %do;
         %let len_excp_msg1=230;
   %end;


   data _null_; 
       call symput('etl_xobsbefore', %di_util_nobs(&etlp_exception_table));
   run;

%mend etl_check_exist;

%etl_check_exist;

/*---- End of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            Lookup_and_validation                                     * 
 * Transform:       Lookup_and_validation                                     * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Tables:   User Written Code Target -                                * 
 *                   DIDM_STG.STG_GEO_PROD_PRICE                              * 
 *                  PROD_HIER_SK - DI_DM.PRODUCT_DM                           * 
 *                  GEOGRAPHY_DM - DI_DM.GEOGRAPHY_DM                         * 
 * Target Table:    work.stg_price_current                                    * 
 *                  stg_current_week_price                                    *
 *============================================================================*/ 


/*---- Map the columns ----*/ 
proc datasets lib = work nolist nowarn memtype = (data view); 
delete vw_etl_lookup_product_dm; 
quit; 

%put ______:  %str(NOTE: Mapping columns ...); 

proc sql noprint; 
create table work.vw_etl_lookup_product_dm as 
select 
(MAX(PROD_HIER_SK)) as PROD_HIER_SK length = 6, 
PROD_ID, 
(MAX(STD_HIER_PROD_LVL)) as STD_HIER_PROD_LVL length = 4 
from di_dm.product_dm 
group by 
PROD_ID 
; 
quit; 


%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.); 

%let runTime = %sysfunc(datetime()); 

proc datasets lib = work nolist nowarn memtype = (data view);
   delete stg_price_current stg_current_week_price stg_prod_price_future;
quit;

%macro price_data_validate;

proc sql noprint;
   select count(*) into: stg_cnt from stg_lib.&src_table;
quit;

%let stg_cnt=&stg_cnt;

%put ______: Number of records in stg_lib.&src_table = &stg_cnt;

   %if &stg_cnt eq 0 %then %do;
   
      /* create empty dataset for next step to consume */
      DATA work.stg_price_current (drop = EXPIRATION_DT)
           work.stg_current_week_price (drop = EXPIRATION_DT)
           work.stg_prod_price_future; 

         attrib PROD_HIER_SK length=6
                GEO_HIER_SK length=4
            /*
                PROD_ID length=$32
                GEO_ID length=$32*/
                EFFECTIVE_DT length=8 format=DATE9.
                EXPIRATION_DT length=8 format=DATE9.
                REGULAR_PRICE length=8
                ACTUAL_PRICE length=8
                PRICE_TYPE length=4
                UPDATED_DTTM length=8 format = DATETIME16.
                ; 
         stop;
      run;   
   
   %end;
   %else %do;
   
      proc datasets lib=work nolist nowarn memtype = (data view);
         delete invalid_prod_geo_in_stg_price stg_prod_price_future;
      quit;   
   
      /*---- Begin DATA step to perform lookups  ----*/ 
      DATA  work.stg_price_current
               ( keep = PROD_HIER_SK GEO_HIER_SK EFFECTIVE_DT
                        REGULAR_PRICE ACTUAL_PRICE PRICE_TYPE UPDATED_DTTM)
            work.stg_current_week_price           
               ( keep = PROD_HIER_SK GEO_HIER_SK EFFECTIVE_DT
                  REGULAR_PRICE ACTUAL_PRICE PRICE_TYPE UPDATED_DTTM)
         work.stg_prod_price_future
            ( keep = PROD_HIER_SK GEO_HIER_SK EFFECTIVE_DT EXPIRATION_DT
                  REGULAR_PRICE ACTUAL_PRICE PRICE_TYPE UPDATED_DTTM)
            work.invalid_prod_geo_in_stg_price
            ( keep = PROD_ID GEO_ID EFFECTIVE_DT REGULAR_PRICE ACTUAL_PERM_PRICE _EXCEPTION_COND); 

         attrib PROD_HIER_SK length=6
                GEO_HIER_SK length=4
                PROD_ID length=$32
                GEO_ID length=$32
                EFFECTIVE_DT length=8 format=DATE9.
                REGULAR_PRICE length=8
                ACTUAL_PRICE length=8
                PRICE_TYPE length=4
                UPDATED_DTTM length=8 format = DATETIME16.
                STD_HIER_PROD_LVL length=4
                STD_HIER_GEO_LVL  length=4
                PROD_INTRODUCTION_DT length=8 FORMAT=DATE9. INFORMAT= DATE9.
                _EXCEPTION_COND length=$8
                ; 
      retain missing0-missing2 0; 
         /* Build hash objects from lookup tables before reading first source row  */ 
         if (_n_ = 1) then 
         do; 
         /* Build hash h0 from lookup table DI_MON.PROD_HIER_SK */ 
         %di_util_hash_define( h_num = 0, 
                               dsn   = %QUOTE(work.vw_etl_lookup_product_dm), 
                               d_key = %QUOTE("PROD_ID"), 
                               d_data= %QUOTE("PROD_HIER_SK","STD_HIER_PROD_LVL") 
                               );            

          /* Build hash h1 from lookup table DI_DM.GEOGRAPHY_DM */ 
          %di_util_hash_define( h_num = 1, 
                                dsn   = %QUOTE(DI_DM.GEOGRAPHY_DM ), 
                                d_key = %QUOTE("GEO_ID"), 
                                d_data= %QUOTE("GEO_HIER_SK","STD_HIER_GEO_LVL") 
                                ); 

          /* Build hash h2 from lookup table DI_DM.INVALID_RECYCLED_PRODUCTS */ 

/*
            %di_util_hash_define( h_num = 2, 
                                dsn   = %QUOTE(DI_DM.INVALID_RECYCLED_PRODUCTS ), 
                                d_key = %QUOTE("PROD_ID"), 
                                d_data= %QUOTE("PROD_INTRODUCTION_DT") 
                                );            
*/
/*            call missing (PROD_ID, PROD_HIER_SK, GEO_ID, GEO_HIER_SK, STD_HIER_PROD_LVL, STD_HIER_GEO_LVL, PROD_INTRODUCTION_DT ); */
              call missing (PROD_ID, PROD_HIER_SK, GEO_ID, GEO_HIER_SK, STD_HIER_PROD_LVL, STD_HIER_GEO_LVL); 
         end; /* All hash objects have been defined */

         /* Read a row from the source table  */ 
         set stg_lib.&src_table (rename=(ACTUAL_PERM_PRICE=ACTUAL_PRICE)) end = eof;
         EXPIRATION_DT = "31DEC5999"d;         
         UPDATED_DTTM = &runTime;
         call missing(_EXCEPTION_COND);
        
         /* Is the current key value stored in hash h0?  */ 
         if (missing0 < 1) then 
         do; 
            rc0 = h0.find();
         end; 
         else 
            rc0 = missing0; 

         /* Is the current key value stored in hash h1?  */ 
         if (missing1 < 1) then 
         do; 
            rc1 = h1.find();
         end; 
         else 
            rc1 = missing1; 

         /* Is the current key value stored in hash h1?  */ 
/*
         if (missing2 < 1) then 
         do; 
            rc2 = h2.find();
         end; 
         else 
            rc2 = missing2; 
*/
         if rc0 ne 0 then do;
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'P');
            call missing (PROD_HIER_SK);   
         end;
         
         if rc1 ne 0 then do;
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'G');         
            call missing (GEO_HIER_SK); 
         end;
/*
         if rc2 eq 0 then do;
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'R');         
            call missing (PROD_INTRODUCTION_DT); 
         end;
*/         
         if rc0=0 and STD_HIER_PROD_LVL ne &l_prod_lvl then
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'p');
         
         if rc1=0 and STD_HIER_GEO_LVL ne &l_geo_lvl then
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'g');
            
         if effective_dt gt &wk_end_dt and regular_price gt 0 and actual_price le 0 then
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'A');         
         
       if effective_dt gt &wk_end_dt and regular_price le 0 and actual_price gt 0 then
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'R');         

         if effective_dt le &wk_end_dt and regular_price le 0 then
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'r');
            
         if effective_dt le &wk_end_dt and actual_price le 0 then 
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'a');

         if regular_price lt actual_price then 
            _EXCEPTION_COND = cats(_EXCEPTION_COND,'l');

         if (regular_price ne 0 and actual_price ne 0) and (regular_price eq actual_price)then 
            price_type=1;

         if (regular_price ne 0 and actual_price ne 0) and (regular_price gt actual_price)then 
            price_type=3;

      /* split into three tables, each for Current, Future and History */
        if _EXCEPTION_COND eq '' then do;
          if effective_dt gt &wk_end_dt then
               output work.stg_prod_price_future;
          if (effective_dt ge &wk_start_dt) and ( effective_dt le &wk_end_dt) then
               output work.stg_current_week_price;
          if effective_dt lt &wk_start_dt then
               output work.stg_price_current;
       end; 
       else do;
            exception_total + 1;
            ACTUAL_PERM_PRICE=ACTUAL_PRICE; 
            output work.invalid_prod_geo_in_stg_price;
       end;   

       if (eof = 1) then do; 
           put "Total lookup exceptions: " exception_total ; 
       end; 

      run;
      
      %rcSet(&syserr);
      
      proc datasets lib=work nolist nowarn memtype = (data view);
         delete temp_prod_price_x;
      quit;

      data work.temp_prod_price_x;
         set work.invalid_prod_geo_in_stg_price;
         attrib ETLS_EXCEPTION_COND length=$&len_excp_msg1;
         attrib ETL_ERROR_JOBRUNTIME length=8 format=datetime16.;
      
         ETL_ERROR_JOBRUNTIME = "&etls_startTime"dt;
      
         if index(_EXCEPTION_COND,'p') then 
	 ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5353,args=PROD_ID)");     

         if index(_EXCEPTION_COND,'g') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5353,args=GEO_ID)");      

         if index(_EXCEPTION_COND,'P') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=Prod_ID)"); 

         if index(_EXCEPTION_COND,'G') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=Geo_ID)"); 
            
         if index(_EXCEPTION_COND,'r') or index(_EXCEPTION_COND,'R') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5331,args=Regular_price 0)");

         if index(_EXCEPTION_COND,'a') or index(_EXCEPTION_COND,'A') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5331,args=Actual_price 0)");

         if index(_EXCEPTION_COND,'l') then 
            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5328,args=Regular_Price Actual_Price)");
/* invalid recycle condition*/
/*         if index(_EXCEPTION_COND,'R') then */
/*            ETLS_EXCEPTION_COND = cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5356)");*/

         drop _EXCEPTION_COND;
      run; 

      %rcSet(&syserr);
      
      proc append base=err_lib.&err_table 
                  data=work.temp_prod_price_x force;
      run;

      %rcSet(&syserr);
   
   %end;

%mend price_data_validate;

%price_data_validate;

/*============================================================================*
 * Step:            UW Remove duplicate records                               *
 * Transform:       User Written                                              *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    work.stg_price_current                                    *
 *                  work.stg_current_week_price                               *
 * Target Table:    User Written Code Target -                                *
 *                   work.stg_prod_price_sorted                               *
 *                   work.stg_current_week_price_sorted                       * 
 *                                                                            *
 * User Written:    Sort                                                      *
 *============================================================================*/

%di_etl_keep_work_table(stg_price_current stg_current_week_price stg_prod_price_future vw_etl_lookup_product_dm);

/*  Remove duplicate from both stg_price_current and stg_current_week_price stg_prod_price_future*/

proc datasets lib=work nolist nowarn memtype = (data view);
/*   delete stg_prod_price_sorted pf_dups stg_current_week_price_sorted curr_wk_dups;*/
delete stg_prod_price_sorted pf_dups stg_current_week_price_sorted curr_wk_dups Stg_prod_price_future_sorted pff_dups;
quit;

proc sort data=stg_price_current
          out=work.stg_prod_price_sorted (keep=geo_hier_sk prod_hier_sk effective_dt
                   price_type actual_price regular_price updated_dttm)
          nodupkey
        dupout=work.pf_dups (keep=geo_hier_sk prod_hier_sk effective_dt  price_type
                 actual_price regular_price);
     by prod_hier_sk geo_hier_sk effective_dt;
run;

%rcSet(&syserr);

proc sort data=stg_current_week_price
          out=stg_current_week_price_sorted(keep=geo_hier_sk prod_hier_sk effective_dt
                   price_type actual_price regular_price updated_dttm)
          nodupkey
         dupout=work.curr_wk_dups (keep=geo_hier_sk prod_hier_sk effective_dt  price_type
                 actual_price regular_price);
     by prod_hier_sk geo_hier_sk effective_dt;
run;

%rcSet(&syserr);

proc sort data=stg_prod_price_future
          out=Stg_prod_price_future_sorted(keep=geo_hier_sk prod_hier_sk effective_dt expiration_dt 
                   price_type actual_price regular_price updated_dttm)
          nodupkey
        dupout=work.pff_dups (keep= geo_hier_sk prod_hier_sk effective_dt  price_type
                 actual_price regular_price);
     by prod_hier_sk geo_hier_sk effective_dt;
run;

%rcSet(&syserr);

/* Delete Exception View */
proc datasets lib = work nolist nowarn memtype = (data view);
   delete etls_exception_pf_dups etls_exception_curr_wk_dups etls_exception_pff_dups etls_exception;
quit;

/* Move dupplicate records to exception table */
/* Add etl_error_jobruntime variable to exception table. Also fetch the values of Prod_id and Geo_id corresponding
   to Prod_hier_sk and geo_hier_sk*/

proc sql noprint;
   create table work.etls_exception_pf_dups as
   (
   select g.geo_id length=32,
          p.prod_id length=32,
          effective_dt,
          regular_price,
          actual_price as actual_perm_price,
        "%etl_msg(mid=5351)" as etls_exception_cond length=&len_excp_msg1.,
          "&etls_startTime"dt as etl_error_jobruntime 
     from work.pf_dups x,
          work.vw_etl_lookup_product_dm p,
          di_dm.geography_dm g
    where x.prod_hier_sk = p.prod_hier_sk
      and x.geo_hier_sk = g.geo_hier_sk
     )
;
quit;

proc sql noprint;
   create table work.etls_exception_curr_wk_dups as
   (
   select g.geo_id length=32,
          p.prod_id length=32,
          effective_dt,
          regular_price,
          actual_price as actual_perm_price,
        "%etl_msg(mid=5351)" as etls_exception_cond length=&len_excp_msg1.,
          "&etls_startTime"dt as etl_error_jobruntime 
     from work.curr_wk_dups x,
          work.vw_etl_lookup_product_dm p,
          di_dm.geography_dm g
    where x.prod_hier_sk = p.prod_hier_sk
      and x.geo_hier_sk = g.geo_hier_sk
     )
;
quit;
proc sql noprint;
   create table work.etls_exception_pff_dups as
   (
   select g.geo_id length=32,
          p.prod_id length=32,
          effective_dt,
          regular_price,
          actual_price as actual_perm_price,
        "%etl_msg(mid=5351)" as etls_exception_cond length=&len_excp_msg1.,
          "&etls_startTime"dt as etl_error_jobruntime 
     from work.pff_dups x,
          work.vw_etl_lookup_product_dm p,
          di_dm.geography_dm g
    where x.prod_hier_sk = p.prod_hier_sk
      and x.geo_hier_sk = g.geo_hier_sk
     )
;
quit;

data work.etls_exception
     (keep=%di_util_column_names(err_lib.&err_table));
   attrib ETLS_EXCEPTION_COND length=$&len_excp_msg1;
   attrib  ETL_ERROR_JOBRUNTIME length=8 format=datetime16.;
   set work.etls_exception_pf_dups work.etls_exception_curr_wk_dups work.etls_exception_pff_dups;
   etls_exception_cond = "%etl_msg(mid=5351)";
   ETL_Error_JobRunTime = "&etls_startTime"dt;
run;

%rcSet(&syserr); 
%rcSet(&sqlrc);

/* save exceptions */
proc append base=err_lib.&err_table
            data=work.etls_exception force;
run;

/*---- End of User Written Code  ----*/

/*============================================================================*
 * Step:            Purge History                                             *
 * Transform:       Table Loader (Version 2.1)                                *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    User Written Code Target -                                *
 *                   work.stg_prod_price_sorted                               *
 * Target Table:    PRICE_FACT_CURRENT -                                      *
 *                   DI_DM.PRICE_FACT_CURRENT                                 *
 *                                                                            *
 * User Written:    UW Loader                                                 *
 *============================================================================*/

%di_etl_keep_work_table(stg_prod_price_sorted stg_current_week_price_sorted Stg_prod_price_future_sorted vw_etl_lookup_product_dm);

%macro purge_history;
%put ______: ETLP_RELOAD_HISTORY_DATA=&ETLP_RELOAD_HISTORY_DATA;
   %if &ETLP_RELOAD_HISTORY_DATA = 1 %then %do;

      proc datasets lib = work nolist nowarn memtype = (data view);
         delete stg_min_effective_dt stg_prod_price_val;
      quit;

      /* Find out  min(effective_dt) for every Geo/Prod in stage for history*/
      proc sql noprint;
         create table stg_min_effective_dt as
         select prod_hier_sk,
                geo_hier_sk,
                min(effective_dt) as EFFECTIVE_DT format=date9.
           from stg_prod_price_sorted
          group by prod_hier_sk, geo_hier_sk
          order by prod_hier_sk, geo_hier_sk;
      quit;

      %rcSet(&syserr);
      %rcSet(&sqlrc);

      /* Remove price from PRICE_FACT_CURRENT for a Geo/Prod if present in stage and
         PRICE_FACT_CURRENT.valid_from_dt >= stage.effective_dt */

      data trg_lib.PRICE_FACT_CURRENT(keep=%di_util_column_names(trg_lib.PRICE_FACT_CURRENT));
         merge trg_lib.PRICE_FACT_CURRENT(in=dm)
               stg_min_effective_dt (in=stg);
         by prod_hier_sk geo_hier_sk;

         if dm and (NOT stg or valid_from_dt lt effective_dt);
      run;

      %rcSet(&syserr);

      /* Remove price from PRICE_FACT_HISTORY for a Geo/Prod if present in stage and
         PRICE_FACT_HISTORY.valid_from_dt >= stage.effective_dt */

      data trg_lib.PRICE_FACT_HISTORY(keep=%di_util_column_names(trg_lib.PRICE_FACT_HISTORY));
         merge trg_lib.PRICE_FACT_HISTORY(in=dm)
               stg_min_effective_dt (in=stg);
         by prod_hier_sk geo_hier_sk ;

         if dm and (NOT stg or valid_from_dt lt effective_dt);
      run;

      %rcSet(&syserr);

      proc sql noprint;
         alter table stg_prod_price_sorted add min_hist_valid_from_dt DATE format=DATE9.;
      quit;

      %rcSet(&syserr);
      
      proc datasets nolist nodetails lib=work;
         change stg_prod_price_sorted=stg_prod_price_val;
      quit;

      %rcSet(&syserr);
      
   %end;
   %else %do;

      /* This code make sure that if Geo/Prod is present in History/Current Price fact & also present in
       * stage then stage record should have
       * EFFECTIVE_DT >= price_fact_current.valid_from_dt or EFFECTIVE_DT < price_fact_history.valid_from_dt
       * This code is not required if ETLP_RELOAD_HISTORY_DATA=1 because all stage records will
       * override target records
       */

      proc datasets lib = work nolist nowarn memtype = (data view);
         delete stg_prod_price_val;
      quit;

      data stg_prod_price_val;
         attrib min_hist_valid_from_dt format=date9.;
         retain min_hist_valid_from_dt .;
         merge stg_prod_price_sorted(in=stg)
               trg_lib.price_fact_history(in=hist keep=prod_hier_sk geo_hier_sk valid_from_dt
                                  rename=(valid_from_dt=effective_dt));
         by prod_hier_sk geo_hier_sk effective_dt;
         if first.geo_hier_sk then min_hist_valid_from_dt = .;
         if hist and missing(min_hist_valid_from_dt) then min_hist_valid_from_dt = effective_dt;
         if stg ;
      run;
      
      %rcSet(&syserr);
      
   %end;
   
%mend purge_history;

%purge_history;

/*============================================================================*
 * Step:            Move future prices to current                             *
 * Transform:       Table Loader (Version 2.1)                                *
 * Description:                                                               *
 *                                                                            *
 * Source Table:    User Written Code Target -                                *
 *                   work.stg_prod_price_sorted                               *
 * Target Table:    PRICE_FACT_CURRENT -                                      *
 *                   DI_DM.PRICE_FACT_CURRENT                                 *
 *                                                                            *
 * User Written:    UW Loader                                                 *
 *============================================================================*/

%di_etl_keep_work_table(stg_prod_price_val stg_current_week_price_sorted Stg_prod_price_future_sorted vw_etl_lookup_product_dm);

%macro move_future_price_to_current;

   * Decide whether to move previously loaded future data into current;
%put ______: etlp_move_futr_to_curr=&etlp_move_futr_to_curr;
   %if (&etlp_move_futr_to_curr eq 1) %then %do;

      /* Find out current records present in price_fact_future */

      data curr_price_in_future_fact;
        set trg_lib.price_fact_future (where = (valid_from_dt le &wk_end_dt));
        by prod_hier_sk geo_hier_sk;
        drop valid_to_dt;
      run;

      %rcSet(&syserr);
      
      /* current records present in source are in work.stg_current_week_price_sorted /
       *
       * if stg_data_override is 1 then if G/P is present in source then do not bring current week's
       * prices for those G/P from price_fact_future
       * if stg_data_override is 0 then if G/P is present in  both, source & price_fact_future; then
       * discard prices in stage for current week and bring current week prices for those G/P from
       * price_fact_future
       * if a geo-prod-cur_week is not present in stage but in pf_future , then it comes from pf_future
       *
       */

      proc datasets lib = work nolist nowarn memtype = (data view);
         delete stg_plus_future_to_curr_curr_wk;
      quit;

      %if &stg_data_override eq 1 %then
      %do;

         data stg_plus_future_to_curr_curr_wk(rename=new_effective_dt=EFFECTIVE_DT);
            retain last_eff .;
            attrib new_effective_dt length=8 format=DATE9.;            
            merge curr_price_in_future_fact(in=fut)
                  stg_current_week_price_sorted(in=stg);
            by prod_hier_sk geo_hier_sk;
            last_eff=lag(effective_dt);
            if (stg eq 0) or (stg and first.geo_hier_sk and not missing(effective_dt)) or
               (stg and not first.geo_hier_sk and last_eff ne effective_dt);
            new_effective_dt=coalesce(effective_dt, valid_from_dt);
            drop last_eff valid_from_dt effective_dt;
         run;

      %end;

      %else %do;

         data stg_plus_future_to_curr_curr_wk(rename=new_effective_dt=EFFECTIVE_DT);
            retain last_from .;
            attrib new_effective_dt length=8 format=DATE9.;             
            merge stg_current_week_price_sorted(in=stg)
                  curr_price_in_future_fact(in=fut)
                  ;
            by prod_hier_sk geo_hier_sk;
            last_from=lag(valid_from_dt);
            if (fut eq 0) or (fut and first.geo_hier_sk and not missing(valid_from_dt)) or
               (fut and not first.geo_hier_sk and last_from ne valid_from_dt);
            new_effective_dt=coalesce(valid_from_dt,effective_dt);          
            drop last_from valid_from_dt effective_dt;
         run;

      %end;

      %rcSet(&syserr);
      
      /* Add current week data to stg_prod_price_val */
      data work.stg_prod_price_val;
         merge work.stg_prod_price_val
               work.stg_plus_future_to_curr_curr_wk;
         by prod_hier_sk geo_hier_sk effective_dt;
      run;
      
      %rcSet(&syserr);
      
   %end;
   %else %do;
         data work.stg_prod_price_val;
         merge work.stg_prod_price_val
               work.Stg_current_week_price_sorted;
         by prod_hier_sk geo_hier_sk effective_dt;
      run;
   %end;
%mend;

%move_future_price_to_current;

%rcSet(&syserr);

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW_tbl_ldr_price_fact_current                             * 
 * Transform:       Table Loader (Version 2.1)                                * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    User Written Code Target -                                * 
 *                   work.stg_price_current                                   * 
 * Target Table:    PRICE_FACT_CURRENT -                                      * 
 *                   DI_DM.PRICE_FACT_CURRENT                                 * 
 *                                                                            * 
 * User Written:    UW Loader                                                 * 
 *============================================================================*/ 
%di_etl_keep_work_table(stg_prod_price_val Stg_prod_price_future_sorted vw_etl_lookup_product_dm);

/*---- Define load data macro  ----*/

%macro load_price_fact_history;

   %di_etl_keep_work_table(Stg_price_fact_hist temp_new_hist_prices Stg_prod_price_future_sorted vw_etl_lookup_product_dm);

   %if %sysfunc(exist(work.Stg_price_fact_hist)) %then
   %do;
      %if ((%di_util_nobs(work.temp_new_hist_prices) gt 0) or
           (%di_util_nobs(work.Stg_price_fact_hist) gt 0))  %then
      %do;
         data trg_lib.price_fact_history(keep=%di_util_column_names(trg_lib.price_fact_history));
            merge trg_lib.price_fact_history
                  work.temp_new_hist_prices
                  Stg_price_fact_hist (rename= (in_from_dt=VALID_FROM_DT
                                                in_to_dt=VALID_TO_DT
                                                in_reg_price=REGULAR_PRICE
                                                in_act_price=ACTUAL_PRICE
                                                in_price_type=PRICE_TYPE)) ;
            by prod_hier_sk geo_hier_sk valid_from_dt;
         run;
         %rcSet(&syserr);
      %end;
   %end;
   %else %if (%di_util_nobs(work.temp_new_hist_prices) gt 0) %then
   %do;
      data trg_lib.price_fact_history(keep=%di_util_column_names(trg_lib.price_fact_history));
         merge trg_lib.price_fact_history
               work.temp_new_hist_prices;
         by prod_hier_sk geo_hier_sk valid_from_dt;
      run;
      %rcSet(&syserr);
   %end;

   %if (&ETLP_RELOAD_HISTORY_DATA = 1 or
       (%di_util_nobs(work.Stg_price_fact_hist) gt 0))  %then
   %do;
      data trg_lib.price_fact_history(keep=%di_util_column_names(trg_lib.price_fact_history));
         set trg_lib.price_fact_history;
         by prod_hier_sk geo_hier_sk ;
         /* close out any history records if they exist  */ 
         if not last.geo_hier_sk then do;
            obsnum = _n_ + 1;
            set trg_lib.price_fact_history(keep = valid_from_dt
                          rename = (valid_from_dt = etls_closedate))
                point = obsnum;
            valid_to_dt = etls_closedate - 1;
         end; 
         if last.geo_hier_sk then do;
            set trg_lib.price_fact_current(keep = prod_hier_sk geo_hier_sk valid_from_dt
                          rename = (valid_from_dt = etls_closedate))
                key = PRIM_KEY;
         if _iorc_ gt 0 then
         do;
            _error_ =0;
         end;
         else do;
            valid_to_dt = etls_closedate - 1;
         end;
         end;  
      run;
      %rcSet(&syserr); 
   %end;

   %di_util_index_name(
       lib             = trg_lib, 
       table           = price_fact_history, 
       columns         = prod_hier_sk, 
       create_idx_name = prod_hier_sk
    ); 
    
   %rcSet(&syserr);    

   proc datasets lib=trg_lib nolist nodetails nowarn;
      contents data=price_fact_history out2=work.pfh_tmp_constr noprint;
   quit;

   %local pfh_exist_pk; %let pfh_exist_pk = 0;

   proc sql noprint;
      select count(1) into :pfh_exist_pk from
      (select name from work.pfh_tmp_constr where  upcase(name)="PRIM_KEY" );
   quit;

   %let pfh_exist_pk = &pfh_exist_pk;

   %if %eval(&pfh_exist_pk = 0) %then %do;
      /* creates Primary Key on price_fact_history */
      proc sql noprint;
         alter table trg_lib.price_fact_history add CONSTRAINT PRIM_KEY PRIMARY KEY ( PROD_HIER_SK,GEO_HIER_SK, VALID_FROM_DT);
      quit;
   %end;    
   
   %rcSet(&syserr); 
   %rcSet(&sqlrc); 
   
%mend load_price_fact_history;

%macro etls_loader_price_fact;

   /*---- Close out any history records in source  ----*/
   data stg_prod_price_continuous;
      attrib last_start length=8 format = DATE9.;
      attrib EXPIRATION_DT length=8 format = DATE9. row_output length=$1;
      retain last_start . row_output '';
      set stg_prod_price_val;
      by prod_hier_sk geo_hier_sk;
      if first.geo_hier_sk or last_start=. or row_output = 'Y' then last_start=effective_dt;
      if not last.geo_hier_sk then do;
         obsnum = _n_ + 1;
         set stg_prod_price_val(keep = effective_dt actual_price regular_price price_type
                             rename = (effective_dt = next_eff_dt 
                                       actual_price=next_actual_price
                                       regular_price = next_regular_price
                                       price_type = next_price_type))
                             point = obsnum;

         if (actual_price ne next_actual_price or
             regular_price ne next_regular_price or
             price_type ne next_price_type) then do;

            effective_dt = last_start;
            expiration_dt = next_eff_dt - 1;
            row_output = 'Y';
         end;
         else do;/* actual_price are same */
            effective_dt = last_start;
            row_output = 'N';
         end;
      end;
      else do;
         if row_output = 'Y' then do;
            expiration_dt='31DEC5999'd;
            row_output = 'Y';
         end;
         else do;
            effective_dt = last_start;
            expiration_dt='31DEC5999'd;
            row_output = 'Y';
         end;
      end;
      drop last_start next_eff_dt next_actual_price  row_output
           next_regular_price next_price_type;
      if row_output = 'Y';
    run;

   %rcSet(&syserr);
   %rcSet(&sqlrc);

   /*---- Update master table  ----*/ 
   %put ______: NOTE: Updating table ...;
   %if &etls_recordsBefore_pfc eq 0 %then /*INITIAL LOAD*/
   %do;
      /* - Remove constraints and indexes
         - Append data to this table from stage table
      */
      /* delete indexes and constraints */
      %etl_drop(trg_lib.price_fact_current,ic);
      %etl_drop(trg_lib.price_fact_current,index);      
      %rcSet(&syserr); 

      /*---- Map the columns  ----*/ 
      proc datasets lib = work nolist nowarn memtype = (data view);
         delete mapped;
      quit;
      
      %put ______: NOTE: Mapping columns ...;
      data work.mapped(keep=%di_util_column_names(trg_lib.price_fact_current))
            /view=work.mapped;
         set trg_lib.price_fact_current(obs=0) 
             stg_prod_price_continuous;
             if expiration_dt ge &wk_end_dt;
             VALID_FROM_DT = EFFECTIVE_DT;
             VALID_TO_DT = EXPIRATION_DT;
      run;
      
      /* Append data in Price_fact_current */ 
      proc append force
         base=TRG_LIB.PRICE_FACT_CURRENT
         data=work.mapped;
      run;

      /*---- Map the columns  ----*/ 
      proc datasets lib = work nolist nowarn memtype = (data view);
         delete temp_new_hist_prices;
      quit;

      /* Create target table for price_fact_history */
      data work.temp_new_hist_prices(keep=%di_util_column_names(trg_lib.price_fact_history));
         set trg_lib.price_fact_history(obs=0) 
             stg_prod_price_continuous;
             if expiration_dt lt &wk_end_dt;
             VALID_FROM_DT = EFFECTIVE_DT;
             VALID_TO_DT = EXPIRATION_DT;
      run;      

      %rcSet(&syserr); 
      %rcSet(&sqlrc); 
      
   %end; /* End of Initial load */
   %else
   %do;  /* Incremental load */

      proc datasets lib = work nolist nowarn memtype = (data view);
         delete temp_new_hist_prices price_fact_current_x price_fact_current_old stg_price_fact_hist;
      quit;

      /* Process data only if valid records in source */

      %if %di_util_nobs(work.stg_prod_price_continuous) gt 0 %then %do;

         /* delete indexes and constraints */
         %etl_drop(trg_lib.price_fact_current,ic);
         %etl_drop(trg_lib.price_fact_current,index);      
         %rcSet(&syserr); 

         /* Get the variable list in target table */
         /* use match-merge to update             */
         
         data trg_lib.price_fact_current (keep=%di_util_column_names(trg_lib.price_fact_current))
              work.temp_new_hist_prices (keep=%di_util_column_names(trg_lib.price_fact_history))
              work.stg_price_fact_hist (keep=geo_hier_sk prod_hier_sk in_from_dt in_to_dt
                                              in_reg_price in_act_price in_price_type)
              work.price_fact_current_x (keep=geo_hier_sk prod_hier_sk in_from_dt in_to_dt
                                              in_reg_price in_act_price in_price_type);;
            attrib price_change length=3;
            retain first_hist_rec 0;
            merge trg_lib.price_fact_current(in=inf)
                  stg_prod_price_continuous(in=ins
                          rename=(effective_dt = in_from_dt 
                                  expiration_dt = in_to_dt 
                                  regular_price = in_reg_price 
                                  actual_price = in_act_price
                                  price_type = in_price_type
                                  updated_dttm = in_upd_dttm));
            by prod_hier_sk geo_hier_sk;
            call missing(price_change);
            /* Start: G/P present in both, source & Target */
            if inf and ins then do;
               if first.geo_hier_sk then first_hist_rec = 1;
               /* Start: STAGE.EFFECTIVE_DT < CURRENT.VALID_FROM_DT */
               if (in_from_dt lt valid_from_dt) then do;
                  /* in normal load, stage data cannot have any overlap period with hist+current facts */
                  if (in_from_dt ge min_hist_valid_from_dt and min_hist_valid_from_dt gt 0) then
                     output work.price_fact_current_x;
                  /* If not the last record in stage then insert into history*/
                  else
                     output work.stg_price_fact_hist;
                  if last.geo_hier_sk then do;
                     if valid_to_dt ge &wk_end_dt then
                        output trg_lib.price_fact_current;
                     else output work.temp_new_hist_prices;
                  end;
               end;
               /* End: STAGE.EFFECTIVE_DT < CURRENT.VALID_FROM_DT */
               else if (in_from_dt eq valid_from_dt) then do;
                  if ((regular_price ne in_reg_price) or (actual_price ne in_act_price)
                     or (price_type ne in_price_type)) then do;
                     regular_price = in_reg_price;
                     actual_price = in_act_price;
                     price_type = in_price_type;
                     updated_dttm = in_upd_dttm;
                  end;
                  valid_from_dt = in_from_dt;
                  valid_to_dt = in_to_dt;
                  if valid_to_dt ge &wk_end_dt then
                     output trg_lib.price_fact_current;
                  else do;
                     first_hist_rec = 0;
                     output work.temp_new_hist_prices;
                  end;
               end;
               else do;/* in_from_dt gt valid_from_dt */
                  if first_hist_rec = 1 then do;
                     first_hist_rec = 0;
                     if ((regular_price ne in_reg_price) or (actual_price ne in_act_price)
                        or (price_type ne in_price_type)) then do;
                        valid_to_dt = in_from_dt -1;
                        output work.temp_new_hist_prices;
                     end;
                     else do;
                        valid_to_dt = in_to_dt;
                        price_change = 0;
                     end;
                  end;
                  if first_hist_rec = 0 and missing(price_change) then do;
                     valid_from_dt = in_from_dt;
                     valid_to_dt = in_to_dt; 
                     regular_price = in_reg_price; 
                     actual_price = in_act_price;
                     price_type = in_price_type;
                     updated_dttm = in_upd_dttm;
                  end;
                  if valid_to_dt ge &wk_end_dt then
                     output trg_lib.price_fact_current;
                  else output work.temp_new_hist_prices;
               end;
            end;
            /* End: G/P present in both, source & Target */
            else if (ins and not inf) then do;
               valid_from_dt = in_from_dt;
               valid_to_dt = in_to_dt; 
               regular_price = in_reg_price; 
               actual_price = in_act_price;
               price_type = in_price_type;
               updated_dttm = in_upd_dttm;
               if valid_to_dt ge &wk_end_dt then
                  output trg_lib.price_fact_current;
               else output work.temp_new_hist_prices;
            end;
            else do;
               if valid_to_dt ge &wk_end_dt then
                  output trg_lib.price_fact_current;
               else output work.temp_new_hist_prices;
            end;
            drop price_change first_hist_rec;     
         run;
                
         %rcSet(&syserr); 
         %rcSet(&sqlrc);

         /* add ETL_Error_JobRunTime variable to error table  */
         proc sql noprint;
            create table work.price_fact_current_old as
            select g.geo_id length=32,
                   p.prod_id length=32,
                   in_from_dt as effective_dt,
                   in_reg_price as regular_price,
                   in_act_price as actual_price,
                   in_price_type as price_type,
                   "%etl_msg(mid=5352,args=Effective_dt history.valid_from_dt)" as etls_exception_cond length=&len_excp_msg1,
                  datetime() as etl_error_jobruntime 
              from work.price_fact_current_x x,
                work.vw_etl_lookup_product_dm p,
/*                   di_dm.product_dm p,*/
                   di_dm.geography_dm g
             where x.prod_hier_sk = p.prod_hier_sk
               and x.geo_hier_sk = g.geo_hier_sk;
         quit;

         /* save exceptions */
         proc append force 
              base = err_lib.&err_table
              data = work.price_fact_current_old(rename=(actual_price=actual_perm_price) drop=price_type);
         run;
      
      %end; /* If stage is not empty */
      
   %end; /* End of Incremental load */

   %rcSet(&syserr); 

   /* creates constraints and indexes on price_fact_current */
   %di_util_index_name(
       lib             = trg_lib, 
       table           = price_fact_current, 
       columns         = prod_hier_sk, 
       create_idx_name = prod_hier_sk
    ); 
    
   %rcSet(&syserr); 
   %rcSet(&sqlrc); 

      proc datasets lib=trg_lib nolist nodetails nowarn;
      contents data=price_fact_current out2=work.pf_tmp_constr noprint;
   quit;

   %local pfc_exist_pk; %let pfc_exist_pk = 0;

   proc sql noprint;
      select count(1) into :pfc_exist_pk from
      (select name from work.pf_tmp_constr where upcase(name)="PRIM_KEY" );
   quit;

   %let pfc_exist_pk = &pfc_exist_pk;

   %if %eval(&pfc_exist_pk = 0) %then %do;
      /* creates Primary Key on Price_fact_current */
      proc sql noprint;
         alter table trg_lib.price_fact_current add CONSTRAINT PRIM_KEY PRIMARY KEY ( PROD_HIER_SK,GEO_HIER_SK);
      quit;
   %end;

   /*============================================================================* 
    * Step:            UW_tbl_ldr_price_fact_history                             * 
    * Transform:       Table Loader (Version 2.1)                                * 
    * Description:                                                               * 
    *                                                                            * 
    * Source Table:    PRICE_FACT_CURRENT -                                      * 
    *                   DI_DM.PRICE_FACT_CURRENT                                 * 
    * Target Table:    PRICE_FACT_HISTORY -                                      * 
    *                   DI_DM.PRICE_FACT_HISTORY                                 * 
    *                                                                            * 
    * User Written:    Merge Loader                                              * 
    *============================================================================*/ 
   /* Insert expired prices in price_fact_history */ 

   /*---- Update master table using match merge to preserve sort order ----*/

   %load_price_fact_history;
   
   %if %symexist(part_sk) %then %do;
      %let etls_lib=d_&part_sk;
      %put ______: etls_lib=&etls_lib;
   %end;
   %else
      %let etls_lib=di_dm;

%mend etls_loader_price_fact;

%etls_loader_price_fact;

/*---- End of User Written Code  ----*/ 

/* Load Price_Fact_Future*/

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            Load_price_fact_future                                    * 
 * Source Table:    stg_prod_price_future -                                   * 
 *                   work.stg_prod_price_future                               * 
 * Target Table:    PRICE_FACT_FUTURE -                                       * 
 *                   DI_DM.PRICE_FACT_FUTURE                                  * 
 *                                                                            * 
 * User Written:    etl_loader_match_merge                                    * 
 *============================================================================*/ 

%di_etl_keep_work_table(Stg_prod_price_future_sorted vw_etl_lookup_product_dm);

%let etls_reccheckexist = 0;
%let etls_recordsbefore =%di_util_nobs(TRG_LIB.PRICE_FACT_FUTURE);
%let etls_table = %nrquote(PRICE_FACT_FUTURE);

%macro etls_loader_ppf;

/*   proc sort data=stg_prod_price_future nodupkey noequals*/
/*             dupout = work.price_fact_future_dups;*/
/*      by prod_hier_sk geo_hier_sk effective_dt;*/
/*   run;*/

   %rcSet(&syserr); 
   
   /* Insert duplicate records into Error  */ 
   
/*   data WORK.price_fact_future_dups(keep=GEO_ID PROD_ID EFFECTIVE_DT REGULAR_PRICE*/
/*                     ACTUAL_PRICE PRICE_TYPE CHANGE_FLG ETLS_EXCEPTION_COND ETL_ERROR_JOBRUNTIME);*/
/*      set price_fact_future_dups;*/
/*      attrib ETLS_EXCEPTION_COND length=$300;*/
/*      attrib ETL_ERROR_JOBRUNTIME length=8 format=datetime16.;*/
/*      ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5303,args=record)"),$300.);   */
/*      ETL_ERROR_JOBRUNTIME = datetime();*/
/*   run;   */

/*   %rcSet(&syserr); */
/*   */
/*   proc append base = err_lib.&err_table */
/*               data = WORK.price_fact_future_dups force;*/
/*   run;*/
/**/
/*   %rcSet(&syserr); */
%put ______: Future_full_refresh=&Future_full_refresh;

   %if &Future_full_refresh eq 0 %then %do; 
      %if %di_util_nobs(Stg_prod_price_future_sorted) eq 0 %then %do;
         
         data TRG_LIB.PRICE_FACT_FUTURE;
            set TRG_LIB.PRICE_FACT_FUTURE;
            where valid_from_dt gt &wk_end_dt;
         run;       
         
      %end;    
      %else %do;
      
         /* Merge with Price_fact_Future and track changes */

         data TRG_LIB.PRICE_FACT_FUTURE;
            merge TRG_LIB.PRICE_FACT_FUTURE (in = dm)
                  Stg_prod_price_future_sorted (in = stg rename=(effective_dt=valid_from_dt expiration_dt=valid_to_dt
                             actual_price=n_act_price regular_price=n_reg_price price_type=n_price_type
                             updated_dttm=n_upd_dttm));

            by prod_hier_sk geo_hier_sk valid_from_dt;

            if n_act_price ne 0 and n_reg_price  ne 0;
            if valid_from_dt gt &wk_end_dt.; 

            if stg then do;

               if (dm = 0) or 
                  (actual_price ne n_act_price or
                   regular_price ne n_reg_price or
                   price_type ne n_price_type) then do;

                  actual_price=n_act_price;
                  regular_price=n_reg_price;
                  price_type=n_price_type;
                  updated_dttm= n_upd_dttm;
               end;           

            end;
            last_geo_hier_sk=lag(geo_hier_sk);
            last_act_price=lag(actual_price);
            last_reg_price=lag(regular_price);
            last_price_type=lag(price_type);

            if first.geo_hier_sk or last_geo_hier_sk ne geo_hier_sk or 
               last_act_price ne actual_price or
               last_reg_price ne regular_price or
               last_price_type ne price_type;

            drop last_geo_hier_sk last_act_price last_reg_price last_price_type 
                 n_act_price n_reg_price n_price_type n_upd_dttm;
         run;

         %rcSet(&syserr); 

         /* Close out records */
         data TRG_LIB.PRICE_FACT_FUTURE (drop = etls_closedate);

            set TRG_LIB.PRICE_FACT_FUTURE;
            by prod_hier_sk geo_hier_sk;

            /* close out any history records if they exist  */ 
            if not last.geo_hier_sk then do;      
               obsnum = _n_ + 1;
               set TRG_LIB.PRICE_FACT_FUTURE(keep = valid_from_dt
                           rename = (valid_from_dt = etls_closedate))
                   point = obsnum;
               valid_to_dt = etls_closedate - 1;            
            end; 
            else if last.geo_hier_sk then do;
               valid_to_dt = "31DEC5999"d;
            end;
         run;    

         %rcSet(&syserr); 
     
     %end;
   %end;   
   %else %do; /* Full Refresh option set */

      /* Remove records with actual_price=0=regular_price before appending */
      /* Compress data based on changes */
      
      data Val_price_fact_future_compres;

         set Stg_prod_price_future_sorted(rename=(effective_dt=valid_from_dt expiration_dt=valid_to_dt)
                                                  where=(actual_price ne 0 and regular_price ne 0));
         by prod_hier_sk geo_hier_sk;

         last_act_price=lag(actual_price);
         last_reg_price=lag(regular_price);
         last_price_type=lag(price_type);

         if first.geo_hier_sk or 
            last_act_price ne actual_price or
            last_reg_price ne regular_price or
            last_price_type ne price_type;

         drop last_act_price last_reg_price last_price_type;
      run;
      
      %rcSet(&syserr); 
      
      /* Close out records */
      data Val_price_fact_future_compres (drop = etls_closedate);

         set Val_price_fact_future_compres;
         by prod_hier_sk geo_hier_sk;

         /* close out any history records if they exist  */ 
         if not last.geo_hier_sk then do;      
            obsnum = _n_ + 1;
            set val_price_fact_future_compres(keep = valid_from_dt
                        rename = (valid_from_dt = etls_closedate))
                point = obsnum;
            valid_to_dt = etls_closedate - 1;            
         end;
         else if last.geo_hier_sk then do;
            valid_to_dt = "31DEC5999"d;
         end;
      run;

      %rcSet(&syserr); 
      
      /* delete indexes and constraints; they will be recreated after load */
      
      %di_util_truncate_table(trg_lib.price_fact_future);

      %etl_drop(trg_lib.price_fact_future,ic);
      %etl_drop(trg_lib.price_fact_future,index);  

      proc append base = trg_lib.PRICE_FACT_FUTURE
                  data = Val_price_fact_future_compres force;
      quit;

      %rcSet(&syserr); 
       
   %end;
   
%mend;

%etls_loader_ppf;

/*---- End of User Written Code  ----*/ 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW - Post Process                       A5RCIHUB.BC0004KS * 
 * Transform:       Summary Statistics                                        * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    PRICE_FACT_FUTURE -                     A5RCIHUB.BR0002MN * 
 *                   DI_DM.PRICE_FACT_FUTURE                                  * 
 *                                                                            * 
 * User Written:    Post Incremental                                          * 
 *============================================================================*/ 

%macro Post_incrProcess;
   /* recreate indexes and constraints on PRICE_FACT_FUTURE */
   %etl_create2(_SASSchemaFile=&etlp_ddl_path, 
                _tableName=PRICE_FACT_FUTURE, _dataset=0, _constraints=1, _index=0,
                LIBREF=trg_lib,DTTMFMT=DATETIME21., DTFMT=DATE9.)
   
   proc sql noprint;
      create index PROD_HIER_SK on trg_lib.PRICE_FACT_FUTURE(PROD_HIER_SK);
   quit;

   %let etls_recordsAfter = %di_util_nobs(trg_lib.PRICE_FACT_FUTURE);


%if %symexist(part_sk) %then %do;
   %let etls_lib=d_&part_sk;
   %put ______: etls_lib=&etls_lib;
%end;
%else
   %let etls_lib=di_dm;

%mend Post_incrProcess;

%Post_incrProcess;
/*---- End of User Written Code  ----*/

/*============================================================================*/  
/*Get record count from PRICE_FACT_HISTORY;*/
%let etls_recordsAfter_pfh=%di_util_nobs(TRG_LIB.PRICE_FACT_HISTORY);
/* Get record count from PRICE_FACT_CURRENT */
%let etls_recordsAfter_pfc=%di_util_nobs(TRG_LIB.PRICE_FACT_CURRENT);
/* Get record count from PRICE_FACT_FUTURE */
%let etls_recordsAfter_pff=%di_util_nobs(TRG_LIB.PRICE_FACT_FUTURE);


/* record job status for each table */
%let etls_endTime=%sysfunc(putn(%sysfunc(datetime()), datetime21.));

%let etls_table = %nrquote(PRICE_FACT_HISTORY);
%let etls_recordsBefore=&etls_recordsBefore_pfh;
%let etls_recordsAfter =&etls_recordsAfter_pfh;
%etls_sendJobstatus(source_table = stg_lib.&src_table,dataset=err_lib.job_status);

%let etls_table = %nrquote(PRICE_FACT_CURRENT);
%let etls_recordsBefore=&etls_recordsBefore_pfc;
%let etls_recordsAfter =&etls_recordsAfter_pfc;
%etls_sendJobStatus(source_table = stg_lib.&src_table,dataset=err_lib.job_status);

%let etls_table = %nrquote(PRICE_FACT_FUTURE);
%let etls_recordsBefore=&etls_recordsBefore_pff;
%let etls_recordsAfter =&etls_recordsAfter_pff;
%etls_sendJobStatus(source_table = stg_lib.&src_table,dataset=err_lib.job_status);


libname err_lib clear;
libname stg_lib clear;
libname trg_lib clear;
